/*
 * main.c
 *
 * This example is an advanced training material for experienced students.
 *
 * Polling buttons is a wast of performance. Interrupts can be used instead. Doing
 * so is much more difficult, however. Special kinds of routines have to be used
 * and care for interrupts with a higher priority has to be taken.
 *
 * Please see www.freertos.org for more information.
 * xSemaphoreGiveFromISR: FreeRTOS/API Reference/Semaphore / Mutexes
 *
 *  Created: 2017-11-30
 * Modified: 2018-07-23
 *   Author: Wolfgang Neff
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <FreeRTOS.h>
#include <semphr.h>
#include <task.h>

#include "board.h"

#define TASK_PRIORITY (tskIDLE_PRIORITY+1)
#define LED_DELAY pdMS_TO_TICKS(1000)

SemaphoreHandle_t xSemaphore;

void vLEDTask(void* pvParameters)
{
	// Initialize the LED.
    LED_DDR |= (1 << LED1);
    LED_PORT |= (1 << LED1);
	// Super loop of the task.
	while (1) {
		xSemaphoreTake(xSemaphore, portMAX_DELAY);
		LED_PORT &= ~(1 << LED1);
		vTaskDelay(LED_DELAY);
		LED_PORT |= (1 << LED1);
	}
}

int main(void)
{
    // Initialize button and enable port interrupts.
	BUTTON_PORT |= (1 << BUTTON2);
	EICRA = (2 << ISC20);
	EIMSK = (1 << INT2);

    // Create the binary semaphore.
	xSemaphore = xSemaphoreCreateBinary();

	// Create the task.
	xTaskCreate(vLEDTask, "LEDTask", configMINIMAL_STACK_SIZE, NULL, TASK_PRIORITY, NULL);

	// Start the scheduler.
	vTaskStartScheduler();
}

// Interrupt service routine for external interrupt 2.
ISR(INT2_vect)
{
    // See FreeRTOS/API Reference/Semaphore / Mutexes/xSemaphoreGiveFromISR
	// for further information. 
	BaseType_t xHigherPriorityTaskWoken = pdFALSE;
	xSemaphoreGiveFromISR(xSemaphore, &xHigherPriorityTaskWoken);
	portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
}

