/** \file switch.h
*
* \brief Debounce active low push-buttons.
*
* \author    Wolfgang Neff
* \version   1.0
* \date      2017-11-27
*
* \par History
*      Created: 2017-11-27
*/

#ifndef SWITCH_H_
#define SWITCH_H_

#include "board.h"

#define SWITCH0_PRESSED() debounce(&BUTTON_PIN, BUTTON0, 0)
#define SWITCH1_PRESSED() debounce(&BUTTON_PIN, BUTTON1, 1)
#define SWITCH2_PRESSED() debounce(&BUTTON_PIN, BUTTON2, 2)

#ifdef __cplusplus
extern "C"
{
#endif

/// <summary>Debounces active low push-buttons.</summary>
/// <remarks>
/// This routine debounces up to eight active low push-buttons. Each key
/// needs an arbitrary key number. This routine is not thread save.
/// </remarks>
/// <param name="port">Port of the key.</param>
/// <param name="pin">Bit position of the key.</param>
/// <param name="key">Number assigned to the key.</param>
/// <returns>True if key is pressed.</returns>
uint8_t debounce(volatile uint8_t* port, uint8_t pin, uint8_t key);

#ifdef __cplusplus
}
#endif

#endif /* SWITCH_H_ */