/*
 * main.c
 *
 * This example is an advanced training material for experienced students.
 *
 * It is not necessary to create a task to have something done on a regular base.
 * Software timers can be used instead. They are more lightweight than tasks.
 * Software timers can also be use for one time actions in the future which
 * can only be realized difficultly with tasks. This example compares the
 * two approaches.
 *
 * Please see www.freertos.org for more information.
 * xTimerCreate: FreeRTOS/API Reference/Software Timers
 * xTimerStart: FreeRTOS/API Reference/Software Timers
 *
 *  Created: 2017-12-01
 * Modified: 2018-07-23
 *  Author: Wolfgang Neff
 */ 

#include <avr/io.h>
#include <FreeRTOS.h>
#include <task.h>
#include <timers.h>

#include "board.h"

#define TASK_PRIORITY (tskIDLE_PRIORITY+1)
#define LED_DELAY pdMS_TO_TICKS(250)

// Create the timer.
TimerHandle_t xTimer;

// This task lets blink LED0
void vMyTask(void* pvParameters)
{
	// Initialize the LED.
	LED_DDR |= (1 << LED0);
	LED_PORT |= (1 << LED0);
	// Super loop of the task.
	while (1) {
		vTaskDelay(LED_DELAY);
		LED_PORT ^= (1 << LED0);
	}
}

// This timer callback function lets blink LED1
void vTimerCallback(TimerHandle_t pxTimer)
{
	LED_PORT ^= (1 << LED1);
}

int main(void)
{
	// Setup the timer controlled LED.
	LED_DDR = (1 << LED1);
	LED_PORT = (1 << LED1);
	
    // Create a timer with the name "Timer". Call vTimerCallback after
	// LED_DELAY ticks. Reload timer when it expires. The callback
	// function is thus called every LED_DELAY ticks.
	xTimer = xTimerCreate("Timer", LED_DELAY, pdTRUE, NULL, vTimerCallback);
	xTimerStart(xTimer, 0);

	// Create the task.
	xTaskCreate(vMyTask, "LEDTask", configMINIMAL_STACK_SIZE, NULL, TASK_PRIORITY, NULL);

	// Start the scheduler.
	vTaskStartScheduler();
}
